/* $Id$ */

/* 初期化/ファイナライザ関連を定義するモジュール */

module ddddd.init;

public import ddddd.importer.public_;
private import ddddd.importer.private_;

private import ddddd.vars;
private import ddddd.image;

public:


void ddddd_init()
{
  // SDL関連を初期化する
  if (SDL_Init(SDL_INIT_TIMER | SDL_INIT_AUDIO | SDL_INIT_VIDEO | SDL_INIT_JOYSTICK) < 0) {
    throw new Error("SDL init failed");
  }
  if (TTF_Init() < 0) {
    throw new Error("TTF init failed");
  }
  Mix_OpenAudio(22050, AUDIO_S16SYS, 2, 4096);

  if (SDLNet_Init() < 0) {
    throw new Error("SDL_net init failed");
  }

  return;
}

void ddddd_term()
{
  // SDL_ttf終了
  TTF_Quit();

  // SDL_mixer終了
  Mix_HaltMusic();
  //Mix_FreeMusic(...);
  Mix_CloseAudio();

  // SDL_net終了
  SDLNet_Quit();

  // SDL終了
  SDL_Quit();

  // hWndを消す
  ddddd_rw_hwnd = null;

  // depthを消す
  ddddd_rw_depth = 0;

  return;
}

void ddddd_init_wm(char[] caption = null, char[] iconpath = null,
    int icon_transparent_pixel_x = -1, int icon_transparent_pixel_y = -1)
{
  // caption
  if (caption) {
    version (Win32) {
      SDL_WM_SetCaption(toStringz(utf8_sjis(caption)), null);
    }
    else {
      SDL_WM_SetCaption(toStringz(caption), null);
    }
  }
  // icon
  if (iconpath) {
    _ddddd_set_icon(toStringz(iconpath),
        icon_transparent_pixel_x, icon_transparent_pixel_y);
  }
  // hWndを取得しておく
  SDL_SysWMinfo wminfo;
  if (SDL_GetWMInfo(&wminfo)) {
    ddddd_rw_hwnd = cast(void*)wminfo.window;
  }

  return;
}

SDL_Surface *ddddd_init_rw(int width = 320, int height = 240)
{
  // TODO: ここのパラメータ制御はもう少し考える余地がある
  SDL_Surface *rw = SDL_SetVideoMode(width, height, 0, SDL_SWSURFACE);
  if (!rw) {
    throw new Error("SDL_SetVideoMode failed");
  }
  ddddd_rw_depth = rw.format.BitsPerPixel; // depthを記憶しておく

  return rw;
}


private:

void _ddddd_set_icon(char *iconpath, int x, int y)
{
  SDL_Surface* icon = IMG_Load(iconpath);
  if (!icon) return;
  if ((x != -1) && (y != -1))
  {
    if (SDL_LockSurface(icon) != 0) return;
    SDL_SetColorKey(icon, SDL_SRCCOLORKEY,
        ddddd_getPixel(icon, x, y));
    SDL_UnlockSurface(icon);
  }
  SDL_WM_SetIcon(icon, null);
  return;
}


/* vim:set ft=d sw=2 ts=8 sts=2 et cindent: */
