/* $Id$ */

/* クォータービュー関連のモジュール */
/* TODO: あとで書く */
/* 必要なもの:
 * - 45度単位の回転関数
 * - 配列(マップ)の一部を切り出して別の配列にコピーする関数
 * - サーフィス等を保持する構造体
 * - サーフィス操作関数など
 */
/* TODO: もし、msgwindowと共通する部分が多くなりそうなら、
 *       基底クラスとして共通部分をまとめてもよい
 *       (具体的には、キャッシュや窓描画部分やステップ管理等)
 *       最終的にはまとめるが、msgwindowを実装してからまとめる事にする
 *       (qvはあとまわしなので)
 */
/* note:
   クォータービューでの表示は、多重重ね合わせにする必要がある。
   - 床下。落とし穴や崖の側面部分も含む。
     これは、あらゆる表示よりも下として扱われる。
   -- 落とし穴が(表示上)縦に重なる場合を考慮するなら、
      床下内でも、レイヤを複数持つ事が好ましい。
      (そうしないと、落とし穴から落ちた時に、それが別の落とし穴から
       一瞬見えてしまったりする場合が考えられる)
   - 床。
   - 床用カーソル。
   - キャラクタ含む立体物全般。
   - 立体物用カーソル。
   - その他の前景物。
 */

module ddddd.qv;

public import ddddd.importer.public_;
private import ddddd.importer.private_;


public int ddddd_qv_rotate45_size(void *[][] src)
{
  if (!src.length) return 0;
  return src.length + src[0].length - 1;
}

/* 指定された二次元配列を(クォータービュー表示しやすいように)45度回転させる
   例えば、以下のような配列
   0  1  2
   3  4  5
   6  7  8
   9 10 11
   は、次のように回転される(横は0.5単位に伸びる)(-はnull)。
   - - - 0
   - - 3 - 1
   - 6 - 4 - 2
   9 - 7 - 5
   -10 - 8
   - -11
   ※元になる配列は、常に正規の長方形(または正方形)という前提とする。
 */
public void *[][] ddddd_qv_rotate45_map(void *[][] src)
{
  if (!src.length) return [];
  void *[][] result = [];
  /* 最初に大きさを求めて、resultを初期化する
     (前述の仕様より、変換後は必ず正方形になる)
   */
  int max_size = ddddd_qv_rotate45_size(src);
  result.length = max_size;
  for (int iy = 0; iy < max_size; iy++) {
    result[iy].length = max_size;
    for (int ix = 0; ix < max_size; ix++) {
      result[iy][ix] = null;
    }
  }

  int i = 0;
  int offset = src.length - 1; // (y軸の大きさ-1)
  while (1) {
    if (max_size <= i) break;
    for (int x = 0; x <= i; x++) {
      if (src.length <= (i-x)) continue;
      if (src[0].length <= x) continue;
      if (max_size <= offset+(x*2)) continue;
      result[i][offset+(x*2)] = src[i-x][x];
    }
    offset--;
    i++;
  }
  return result;
}



/* 指定された座標値を、ddddd_qv_rotate45_map()で回転された座標に
   対応した座標値[ y, x ]へと変換する
 */
public int[] ddddd_qv_rotate45_location(int location_x, int location_y,
    void *[][] src)
{
  return ddddd_qv_rotate45_location(location_x, location_y, src.length);
}
public int[] ddddd_qv_rotate45_location(int location_x, int location_y,
    int height)
{
  int[] result;
  result.length = 2;
  result[1] = location_x + height - 1 - location_y; // x
  result[0] = location_x + location_y; // y
  return result;
}

/* 指定されたqv座標値等から、実ドット座標値[ y, x ]に変換する
 */
public int[] ddddd_qv_2bitmap_locate(int qv_x, int qv_y,
    int tile_width, int tile_height)
{
  int[] result;
  result.length = 2;
  result[0] = qv_x*tile_width/2;
  result[1] = qv_y*tile_height/2;
  return result;
}

/* 指定されたマップの座標値のデータを取得する。
   配列の外であっても安全にnullが返される。
   TODO: 折り返し有りバージョンも必要かどうか考える */
public void *peek_map(void*[][] map, int x, int y)
{
  if (map.length <= y) return null;
  if (map[y].length <= x) return null;
  return map[y][x];
}








/* vim:set ft=d sw=2 ts=8 sts=2 et cindent: */
